﻿using System;
namespace State.Ejm1
{
    /// <summary>
    /// Estado sobregiro. Cuando el balance es negativo pero con el limite de sobregiro
    /// </summary>
    public class OverDrawnState : State
    {
        public void SendMailToAccountHolder()
        {
            Console.WriteLine("Attention: Your Account is Overdrawn");
        }

        public OverDrawnState(BusinessAccount account):base(account)
        {
            SendMailToAccountHolder();
        }

        public OverDrawnState(State source):base(source)
        {
            SendMailToAccountHolder();
        }

        public override State TransitionState()
        {
            double balance = GetContext().GetBalance();
            if (balance >= BusinessAccount.MinBalance)
                GetContext().SetState(new NoTransactionFeeState(this));
            else if (balance >= 0)
                GetContext().SetState(new TransactionFeeState(this));
            return GetContext().GetState();
        }

        public override bool Deposit(double amount)
        {
            double balance = GetContext().GetBalance();
            GetContext().SetBalance(balance - BusinessAccount.TransFeeOverdraw);
            Console.WriteLine("Transaction Fee was charged "
                    + "due to account status(Overdrawn)");
            return base.Deposit(amount);
        }

        public override bool Withdraw(double amount)
        {
            double balance = GetContext().GetBalance();
            if ((balance - BusinessAccount.TransFeeOverdraw - amount) > BusinessAccount.OvedrawLimit)
            {
                GetContext().SetBalance(
                        balance - BusinessAccount.TransFeeOverdraw);
                Console.WriteLine("Transaction Fee was charged due to "
                        + "account status(Overdrawn)");
                return base.Withdraw(amount);
            }
            else
            {
                Console.WriteLine(BusinessAccount.ErrOverdrawLimitExceed);
                return false;
            }
        }
    }
}
