﻿using System;
namespace State.Ejm1
{
    /// <summary>
    ///  Estado de la cuenta: con arancel, con gratificacion.
    ///  Cuando el balance de la cuenta es positivo, pero menor que el balance minimo
    /// </summary>
    public class TransactionFeeState : State
    {
        public TransactionFeeState(BusinessAccount account)
            : base(account)
        {

        }

        public TransactionFeeState(State source)
            : base(source)
        {

        }

        public override State TransitionState()
        {
            double balance = GetContext().GetBalance();
            if (balance < 0)
            {
                GetContext().SetState(new OverDrawnState(this));
            }
            else
            {
                if (balance >= BusinessAccount.MinBalance)
                {
                    GetContext().SetState(new NoTransactionFeeState(this));
                }
            }
            return GetContext().GetState();
        }

        public override bool Deposit(double amount)
        {
            double balance = GetContext().GetBalance();
            GetContext().SetBalance(balance - BusinessAccount.TransFeeNormal);
            Console.WriteLine("Transaction Fee was charged due to "
                    + "account status " + "(less than minimum balance)");
            return base.Deposit(amount);
        }

        public override bool Withdraw(double amount)
        {
            double balance = GetContext().GetBalance();
            if ((balance - BusinessAccount.TransFeeNormal - amount) > BusinessAccount.OvedrawLimit)
            {
                GetContext().SetBalance(balance - BusinessAccount.TransFeeNormal);
                Console.WriteLine("Transaction Fee was charged due to "
                        + "account status " + "(less than minimum balance)");
                return base.Withdraw(amount);
            }
            else
            {
                Console.WriteLine(BusinessAccount.ErrOverdrawLimitExceed);
                return false;
            }
        }
    }
}
